# Generated by Django 5.0.1 on 2024-06-07 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_follows", "0004_alter_follow_unique_together_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="follow",
            old_name="new_actor",
            new_name="actor",
        ),
        migrations.RenameField(
            model_name="follow",
            old_name="new_target",
            new_name="target",
        ),
    ]



import graphene
import swapper
from baseapp_core.graphql import RelayMutation, get_obj_from_relay_id, login_required
from django.utils.translation import gettext_lazy as _
from graphql.error import GraphQLError
from graphql_relay import offset_to_cursor

from .interfaces import FollowsInterface

Follow = swapper.load_model("baseapp_follows", "Follow")


class FollowToggle(RelayMutation):
    follow = graphene.Field(
        lambda: Follow.get_graphql_object_type()._meta.connection.Edge, required=False
    )
    target = graphene.Field(FollowsInterface)
    actor = graphene.Field(FollowsInterface)
    follow_deleted_id = graphene.ID(required=False)

    class Input:
        actor_object_id = graphene.ID(required=True)
        target_object_id = graphene.ID(required=True)

    @classmethod
    @login_required
    def mutate_and_get_payload(cls, root, info, **input):
        FollowObjectType = Follow.get_graphql_object_type()

        target = get_obj_from_relay_id(info, input.get("target_object_id"))
        actor = get_obj_from_relay_id(info, input.get("actor_object_id"))

        if not info.context.user.has_perm("baseapp_follows.add_follow_with_profile", actor):
            raise GraphQLError(
                str(_("You don't have permission to perform this action")),
                extensions={"code": "permission_required"},
            )

        if not info.context.user.has_perm("baseapp_follows.add_follow", target):
            raise GraphQLError(
                str(_("You don't have permission to perform this action")),
                extensions={"code": "permission_required"},
            )

        follow, created = Follow.objects.get_or_create(
            actor=actor,
            target=target,
        )

        if not created:
            if not info.context.user.has_perm("baseapp_follows.delete_follow", follow):
                raise GraphQLError(
                    str(_("You don't have permission to perform this action")),
                    extensions={"code": "permission_required"},
                )

            follow_deleted_id = follow.relay_id
            follow.delete()
            target.refresh_from_db()
            actor.refresh_from_db()
            return FollowToggle(target=target, actor=actor, follow_deleted_id=follow_deleted_id)

        target.refresh_from_db()
        actor.refresh_from_db()

        return FollowToggle(
            follow=FollowObjectType._meta.connection.Edge(node=follow, cursor=offset_to_cursor(0)),
            target=target,
            actor=actor,
        )


class FollowsMutations(object):
    follow_toggle = FollowToggle.Field()



import swapper
from django.contrib.auth.backends import BaseBackend

Follow = swapper.load_model("baseapp_follows", "Follow")


class FollowsPermissionsBackend(BaseBackend):
    def has_perm(self, user_obj, perm, obj=None):
        if perm == "baseapp_follows.add_follow":
            # TO DO: check if not blocked?
            return user_obj.is_authenticated
        if perm == "baseapp_follows.add_follow_with_profile":
            return user_obj.has_perm("baseapp_profiles.use_profile", obj)
        if perm == "baseapp_follows.delete_follow":
            if isinstance(obj, Follow):
                return obj.user_id == user_obj.id or user_obj.has_perm(
                    "baseapp_profiles.use_profile", obj.actor
                )



import pytest
import swapper
from baseapp_profiles.tests.factories import ProfileFactory
from django.contrib.auth import get_user_model

Follow = swapper.load_model("baseapp_follows", "Follow")

pytestmark = pytest.mark.django_db

User = get_user_model()


def test_follow_and_unfollow():
    profile1 = ProfileFactory()
    profile2 = ProfileFactory()

    follow = Follow.objects.create(
        actor=profile1,
        target=profile2,
    )

    profile1.refresh_from_db()
    profile2.refresh_from_db()

    assert profile1.following_count == 1
    assert profile2.followers_count == 1

    follow.delete()

    profile1.refresh_from_db()
    profile2.refresh_from_db()

    assert profile1.following_count == 0
    assert profile2.followers_count == 0


def test_target_is_following_back():
    profile1 = ProfileFactory()
    profile2 = ProfileFactory()

    original = Follow.objects.create(
        actor=profile1,
        target=profile2,
    )

    reciprocal = Follow.objects.create(
        actor=profile2,
        target=profile1,
    )

    profile1.refresh_from_db()
    profile2.refresh_from_db()
    original.refresh_from_db()
    reciprocal.refresh_from_db()

    assert profile1.following_count == 1
    assert profile2.followers_count == 1
    assert original.target_is_following_back is True
    assert reciprocal.target_is_following_back is True

    reciprocal.delete()

    profile1.refresh_from_db()
    profile2.refresh_from_db()
    original.refresh_from_db()

    assert profile1.following_count == 1  # profile1 still follows profile2
    assert profile2.following_count == 0  # profile2 no longer follows profile1
    assert profile2.followers_count == 1  # profile2 still has profile1 as a follower
    assert profile1.followers_count == 0  # profile1 has no followers anymore
    assert original.target_is_following_back is False  # profile2 no longer follows back profile1



# Generated by Django 5.0.1 on 2024-06-07 01:01
import django.db.models.deletion
import swapper
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_follows", "0005_rename_new_actor_follow_actor_and_more"),
        swapper.dependency("baseapp_profiles", "Profile"),
    ]

    operations = [
        migrations.AlterField(
            model_name="follow",
            name="actor",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="following",
                to=swapper.get_model_name("baseapp_profiles", "Profile"),
                verbose_name="actor",
            ),
        ),
        migrations.AlterField(
            model_name="follow",
            name="target",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="followers",
                to=swapper.get_model_name("baseapp_profiles", "Profile"),
                verbose_name="target",
            ),
        ),
    ]



from baseapp_core.settings.env import env
from baseapp_core.tests.settings import *  # noqa
from celery.schedules import crontab

APPS_DIR = BASE_DIR

# Application definition
INSTALLED_APPS += [
    "channels",
    "avatar",
    "trench",
    "rest_framework_simplejwt",
    "baseapp_auth",
    "testproject.testapp",
    "baseapp_follows",
    "baseapp_profiles",
    "baseapp_pages",
    "baseapp_reactions",
    "baseapp_reports",
    "baseapp_comments",
    "baseapp_blocks",
]

ROOT_URLCONF = "testproject.urls"

CELERY_BEAT_SCHEDULE = {
    "notify_is_password_expired": {
        "task": "baseapp_auth.tasks.notify_users_is_password_expired",
        "schedule": crontab(hour=7, minute=7),
        "options": {"expires": 60 * 60 * 11},
    },
}

# Auth
AUTH_USER_MODEL = "testapp.User"
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "baseapp_auth.permissions.UsersPermissionsBackend",
    "baseapp_profiles.permissions.ProfilesPermissionsBackend",
    "baseapp_follows.permissions.FollowsPermissionsBackend",
]
PASSWORD_HASHERS = ["django.contrib.auth.hashers.PBKDF2PasswordHasher"]

# Sites
URL = env("URL", "", required=False)
FRONT_URL = env("FRONT_URL", "", required=False)
FRONT_CONFIRM_EMAIL_URL = FRONT_URL + "/confirm-email/{id}/{token}"
FRONT_FORGOT_PASSWORD_URL = FRONT_URL + "/forgot-password/{token}"
FRONT_CHANGE_EMAIL_CONFIRM_URL = FRONT_URL + "/change-email/{id}/{token}"
FRONT_CHANGE_EMAIL_VERIFY_URL = FRONT_URL + "/change-email-verify/{id}/{token}"
FRONT_CHANGE_EXPIRED_PASSWORD_URL = FRONT_URL + "/change-expired-password/{token}"

# IOS Deep Links
IOS_CONFIRM_EMAIL_DEEP_LINK = False
IOS_FORGOT_PASSWORD_DEEP_LINK = False
IOS_CHANGE_EMAIL_DEEP_LINK = False

# Android Deep Links
ANDROID_CONFIRM_EMAIL_DEEP_LINK = False
ANDROID_FORGOT_PASSWORD_DEEP_LINK = False
ANDROID_CHANGE_EMAIL_DEEP_LINK = False

# Phone Numbers
PHONENUMBER_DB_FORMAT = "E164"

# BRANCH.IO
BRANCHIO_KEY = env("BRANCHIO_KEY", "N/A")

# JWT Authentication
SIMPLE_JWT = {
    # It will work instead of the default serializer(TokenObtainPairSerializer).
    "TOKEN_OBTAIN_SERIALIZER": "testproject.testapp.rest_framework.jwt.serializers.MyTokenObtainPairSerializer",
    # ...
}
JWT_CLAIM_SERIALIZER_CLASS = "baseapp_auth.rest_framework.users.serializers.UserBaseSerializer"

# Baseapp Auth
BASEAPP_AUTH_USER_FACTORY = "testproject.testapp.tests.factories.UserFactory"



import factory
import swapper
from django.contrib.contenttypes.models import ContentType


class FollowFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = swapper.load_model("baseapp_follows", "Follow")
        abstract = True

    actor_content_type = factory.LazyAttribute(lambda o: ContentType.objects.get_for_model(o.actor))
    actor_object_id = factory.SelfAttribute("actor.id")

    target_content_type = factory.LazyAttribute(
        lambda o: ContentType.objects.get_for_model(o.target)
    )
    target_object_id = factory.SelfAttribute("target.id")

    target_is_following_back = False

    @factory.post_generation
    def actor(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            self.actor = extracted
            self.actor_content_type = ContentType.objects.get_for_model(extracted)
            self.actor_object_id = extracted.id
            self.save()

    @factory.post_generation
    def target(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            self.target = extracted
            self.target_content_type = ContentType.objects.get_for_model(extracted)
            self.target_object_id = extracted.id
            self.save()



# Generated by Django 5.0.1 on 2024-06-07 00:44
import django.db.models.deletion
import swapper
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_follows", "0001_initial"),
        swapper.dependency("baseapp_profiles", "Profile"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name="follow",
            name="user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="follows",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user",
            ),
        ),
        migrations.AddField(
            model_name="follow",
            name="new_actor",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="follows",
                to=swapper.get_model_name("baseapp_profiles", "Profile"),
                verbose_name="profile",
            ),
        ),
        migrations.AddField(
            model_name="follow",
            name="new_target",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="following",
                to=swapper.get_model_name("baseapp_profiles", "Profile"),
                verbose_name="profile",
            ),
        ),
    ]



# Generated by Django 5.0.1 on 2024-06-07 00:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_follows", "0003_migrate_profiles"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="follow",
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name="follow",
            name="actor_content_type",
        ),
        migrations.RemoveField(
            model_name="follow",
            name="actor_object_id",
        ),
        migrations.RemoveField(
            model_name="follow",
            name="target_content_type",
        ),
        migrations.RemoveField(
            model_name="follow",
            name="target_object_id",
        ),
    ]



import graphene
import swapper
from baseapp_core.graphql import get_object_type_for_model, get_pk_from_relay_id
from graphene import relay
from graphene_django.filter import DjangoFilterConnectionField

Follow = swapper.load_model("baseapp_follows", "Follow")
Profile = swapper.load_model("baseapp_profiles", "Profile")


class FollowsInterface(relay.Node):
    followers = DjangoFilterConnectionField(get_object_type_for_model(Follow))
    following = DjangoFilterConnectionField(get_object_type_for_model(Follow))
    followers_count = graphene.Int()
    following_count = graphene.Int()
    is_followed_by_me = graphene.Boolean(
        profile_id=graphene.ID(required=False),
    )

    def resolve_followers_count(self, info):
        return self.followers_count

    def resolve_following_count(self, info):
        return self.following_count

    def resolve_followers(self, info, **kwargs):
        return self.followers.all()

    def resolve_following(self, info, **kwargs):
        return self.following.all()

    def resolve_is_followed_by_me(self, info, profile_id=None, **kwargs):
        if not info.context.user.is_authenticated:
            return False
        if profile_id:
            pk = get_pk_from_relay_id(profile_id)
            profile = Profile.objects.get_if_member(pk=pk, user=info.context.user)
        else:
            profile = info.context.user.current_profile
        return (
            profile
            and Follow.objects.filter(
                actor_id=profile.id,
                target_id=self.id,
            ).exists()
        )



import swapper
from django.db import migrations


class Migration(migrations.Migration):
    def forwards_func(apps, _):
        Profile = swapper.load_model("baseapp_profiles", "Profile")
        Follow = apps.get_model("baseapp_follows", "Follow")

        for follow in Follow.objects.filter(new_actor__isnull=True, actor_object_id__isnull=False):
            follow.new_actor = Profile.objects.get(
                target_content_type_id=follow.actor_content_type_id,
                target_object_id=follow.actor_object_id,
            )

            follow.new_target = Profile.objects.get(
                target_content_type_id=follow.target_content_type_id,
                target_object_id=follow.target_object_id,
            )

            if follow.actor_content_type.model == "user":
                follow.user_id = follow.actor_object_id

            follow.save(update_fields=["new_actor", "new_target", "user_id"])

    def reverse_func(apps, _):
        Follow = apps.get_model("baseapp_follows", "Follow")

        for follow in Follow.objects.filter(new_actor__isnull=False, actor_object_id__isnull=True):
            follow.actor_content_type = follow.new_actor.target_content_type
            follow.actor_object_id = follow.new_actor.target_object_id

            follow.target_content_type = follow.new_target.target_content_type
            follow.target_object_id = follow.new_target.target_object_id

            follow.save(
                update_fields=[
                    "actor_content_type",
                    "actor_object_id",
                    "target_content_type",
                    "target_object_id",
                ]
            )

    dependencies = [
        ("baseapp_follows", "0002_follow_new_actor_follow_new_target"),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]



import swapper
from baseapp_core.graphql.models import RelayModel
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from model_utils.models import TimeStampedModel


class AbstractBaseFollow(TimeStampedModel, RelayModel):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("user"),
        related_name="follows",
        on_delete=models.SET_NULL,
        null=True,
    )

    actor = models.ForeignKey(
        swapper.get_model_name("baseapp_profiles", "Profile"),
        verbose_name=_("actor"),
        related_name="following",
        on_delete=models.CASCADE,
    )

    target_is_following_back = models.BooleanField(default=False)

    target = models.ForeignKey(
        swapper.get_model_name("baseapp_profiles", "Profile"),
        verbose_name=_("target"),
        related_name="followers",
        on_delete=models.CASCADE,
    )

    class Meta:
        abstract = True

    def __str__(self):
        return "{} followed {}".format(self.actor, self.target)

    def save(self, *args, **kwargs):
        created = self._state.adding
        super().save(*args, **kwargs)  # Save the instance first

        if created:
            self.update_followers_count(self.target)
            self.update_following_count(self.actor)
            self.update_target_is_following_back()  # Check for reciprocal relationship after saving

    def update_target_is_following_back(self):
        # Check if the target is following the actor
        reciprocal_follow_exists = self.__class__.objects.filter(
            actor_id=self.target_id,
            target_id=self.actor_id,
        ).exists()

        self.target_is_following_back = reciprocal_follow_exists
        self.save(update_fields=["target_is_following_back"])

        if reciprocal_follow_exists:
            # Update the reciprocal follow instance
            reciprocal_follow = self.__class__.objects.get(
                actor_id=self.target_id,
                target_id=self.actor_id,
            )
            reciprocal_follow.target_is_following_back = True
            reciprocal_follow.save(update_fields=["target_is_following_back"])

    def delete(self, *args, **kwargs):
        actor = self.actor
        target = self.target
        super().delete(*args, **kwargs)

        self.update_followers_count(target)
        self.update_following_count(actor)
        self.update_target_is_following_back_on_delete()

    def update_target_is_following_back_on_delete(self):
        # Check if the target is following the actor
        reciprocal_follow = self.__class__.objects.filter(
            actor_id=self.target_id,
            target_id=self.actor_id,
        ).first()

        if reciprocal_follow:
            reciprocal_follow.target_is_following_back = False
            reciprocal_follow.save(update_fields=["target_is_following_back"])

    def update_followers_count(self, target):
        target.followers_count = target.followers.count()
        target.save(update_fields=["followers_count"])

    def update_following_count(self, actor):
        actor.following_count = actor.following.count()
        actor.save(update_fields=["following_count"])

    @classmethod
    def get_graphql_object_type(cls):
        from .graphql.object_types import FollowObjectType

        return FollowObjectType


class Follow(AbstractBaseFollow):
    class Meta:
        unique_together = [("actor", "target")]

        swappable = swapper.swappable_setting("baseapp_follows", "Follow")


class FollowableModel(models.Model):
    followers_count = models.PositiveIntegerField(default=0, editable=False)
    following_count = models.PositiveIntegerField(default=0, editable=False)

    class Meta:
        abstract = True



# Generated by Django 4.2.7 on 2024-01-09 20:33
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import swapper
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        swapper.dependency("baseapp_follows", "Follow"),
    ]

    operations = [
        migrations.CreateModel(
            name="Follow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                ("actor_object_id", models.PositiveIntegerField(blank=True, null=True)),
                ("target_is_following_back", models.BooleanField(default=False)),
                (
                    "target_object_id",
                    models.PositiveIntegerField(blank=True, db_index=True, null=True),
                ),
                (
                    "actor_content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="actor_follows",
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "target_content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            options={
                "swappable": swapper.swappable_setting("baseapp_follows", "Follow"),
                "unique_together": {
                    (
                        "actor_content_type",
                        "actor_object_id",
                        "target_content_type",
                        "target_object_id",
                    )
                },
            },
        ),
    ]



import graphene
from baseapp_profiles.graphql.queries import ProfilesQueries

from baseapp_follows.graphql.mutations import FollowsMutations
from testproject.testapp.graphql.queries import UsersQueries


class Query(
    graphene.ObjectType,
    UsersQueries,
    ProfilesQueries,
):
    pass


class Mutation(
    graphene.ObjectType,
    FollowsMutations,
):
    pass


schema = graphene.Schema(query=Query, mutation=Mutation)



import swapper
from django.contrib import admin

Follow = swapper.load_model("baseapp_follows", "Follow")


@admin.register(Follow)
class FollowAdmin(admin.ModelAdmin):
    list_display = ("id", "actor", "target", "created", "target_is_following_back")
    list_filter = ("created",)



import swapper
from baseapp_core.graphql import Node, get_object_type_for_model

Follow = swapper.load_model("baseapp_follows", "Follow")


class FollowQuery:
    follow = Node.Field(get_object_type_for_model(Follow))



from django.apps import AppConfig


class FollowsConfig(AppConfig):
    default = True
    name = "baseapp_follows"
    verbose_name = "BaseApp Follows"
    default_auto_field = "django.db.models.AutoField"



# Generated by Django 5.0.1 on 2024-06-07 01:02

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("baseapp_follows", "0006_alter_follow_actor_alter_follow_target"),
        migrations.swappable_dependency(settings.BASEAPP_PROFILES_PROFILE_MODEL),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="follow",
            unique_together={("actor", "target")},
        ),
    ]



import pytest
import swapper
from baseapp_profiles.tests.factories import ProfileFactory

Follow = swapper.load_model("baseapp_follows", "Follow")

pytestmark = pytest.mark.django_db

FOLLOW_TOGGLE_GRAPHQL = """
mutation FollowToggle($input: FollowToggleInput!) {
  followToggle(input: $input) {
    follow {
      node {
        id
      }
    }
    target {
      id
      followersCount
    }
    actor {
      id
      followingCount
    }
  }
}
"""


def test_anon_cant_follow(graphql_client):
    profile1 = ProfileFactory()
    profile2 = ProfileFactory()

    variables = {
        "input": {
            "actorObjectId": profile1.relay_id,
            "targetObjectId": profile2.relay_id,
        }
    }

    response = graphql_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)

    content = response.json()
    assert content["errors"][0]["message"] == "authentication required"
    assert Follow.objects.count() == 0


def test_user_can_follow(django_user_client, graphql_user_client):
    profile1 = ProfileFactory(owner=django_user_client.user)
    profile2 = ProfileFactory()

    variables = {
        "input": {
            "actorObjectId": profile1.relay_id,
            "targetObjectId": profile2.relay_id,
        }
    }

    response = graphql_user_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)

    content = response.json()
    assert content["data"]["followToggle"]["target"]["followersCount"] == 1
    assert content["data"]["followToggle"]["actor"]["followingCount"] == 1


def test_user_can_unfollow(django_user_client, graphql_user_client):
    profile1 = ProfileFactory(owner=django_user_client.user)
    profile2 = ProfileFactory()

    variables = {
        "input": {
            "actorObjectId": profile1.relay_id,
            "targetObjectId": profile2.relay_id,
        }
    }

    response = graphql_user_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)
    content = response.json()

    assert content["data"]["followToggle"]["target"]["followersCount"] == 1
    assert content["data"]["followToggle"]["actor"]["followingCount"] == 1

    response = graphql_user_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)
    content = response.json()

    assert content["data"]["followToggle"]["target"]["followersCount"] == 0
    assert content["data"]["followToggle"]["actor"]["followingCount"] == 0


def test_user_cant_unfollow_others_follow(django_user_client, graphql_user_client):
    profile1 = ProfileFactory(owner=django_user_client.user)
    profile2 = ProfileFactory()
    profile3 = ProfileFactory()

    variables = {
        "input": {
            "actorObjectId": profile1.relay_id,
            "targetObjectId": profile2.relay_id,
        }
    }

    response = graphql_user_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)
    content = response.json()

    assert content["data"]["followToggle"]["target"]["followersCount"] == 1
    assert content["data"]["followToggle"]["actor"]["followingCount"] == 1

    variables = {
        "input": {
            "actorObjectId": profile3.relay_id,
            "targetObjectId": profile2.relay_id,
        }
    }

    response = graphql_user_client(FOLLOW_TOGGLE_GRAPHQL, variables=variables)
    content = response.json()

    assert content["errors"][0]["message"] == "You don't have permission to perform this action"
    assert content["errors"][0]["extensions"]["code"] == "permission_required"



import django_filters
import graphene_django_optimizer as gql_optimizer
import swapper
from baseapp_core.graphql import DjangoObjectType
from graphene import relay

Follow = swapper.load_model("baseapp_follows", "Follow")


class FollowsFilter(django_filters.FilterSet):
    class Meta:
        model = Follow
        fields = ["target_is_following_back"]


class FollowObjectType(gql_optimizer.OptimizedDjangoObjectType, DjangoObjectType):
    class Meta:
        model = Follow
        fields = "__all__"
        interfaces = (relay.Node,)
        filterset_class = FollowsFilter
